"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _createHash = _interopRequireDefault(require("./createHash"));

var _makeRequest = _interopRequireDefault(require("./makeRequest"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const POLL_INTERVAL = 5000; // 5 secs

const VIEWPORT_PATTERN = /^([0-9]+)x([0-9]+)$/;

async function waitFor({
  requestId,
  endpoint,
  apiKey,
  apiSecret
}) {
  const {
    status,
    result
  } = await (0, _makeRequest.default)({
    url: `${endpoint}/api/snap-requests/${requestId}`,
    method: 'GET',
    json: true
  }, {
    apiKey,
    apiSecret,
    maxTries: 5
  });

  if (status === 'done') {
    return result.map(i => Object.assign({}, i, {
      snapRequestId: requestId
    }));
  }

  await new Promise(r => setTimeout(r, POLL_INTERVAL));
  return waitFor({
    requestId,
    endpoint,
    apiKey,
    apiSecret
  });
}

const MIN_INTERNET_EXPLORER_WIDTH = 400;

class RemoteBrowserTarget {
  constructor(browserName, _ref) {
    let {
      viewport,
      chunks = 1,
      maxHeight
    } = _ref,
        otherOptions = _objectWithoutProperties(_ref, ["viewport", "chunks", "maxHeight"]);

    const viewportMatch = viewport.match(VIEWPORT_PATTERN);

    if (!viewportMatch) {
      throw new Error(`Invalid viewport "${viewport}". Here's an example of a valid one: "1024x768".`);
    }

    const [_, width] = viewportMatch; // eslint-disable-line no-unused-vars

    if (browserName === 'edge' && width < MIN_INTERNET_EXPLORER_WIDTH) {
      throw new Error(`Invalid viewport width for the "edge" target (you provided ${width}). Smallest width it can handle is ${MIN_INTERNET_EXPLORER_WIDTH}.`);
    }

    this.chunks = chunks;
    this.browserName = browserName;
    this.viewport = viewport;
    this.maxHeight = maxHeight;
    this.otherOptions = otherOptions;
  }

  async execute({
    globalCSS,
    assetsPackage,
    staticPackage,
    snapPayloads,
    apiKey,
    apiSecret,
    endpoint,
    pages,
    asyncResults = false,
    targetName
  }) {
    const boundMakeRequest = async ({
      slice,
      chunk,
      pageSlice
    }) => {
      const payloadString = JSON.stringify(_objectSpread(_objectSpread({
        viewport: this.viewport,
        maxHeight: this.maxHeight
      }, this.otherOptions), {}, {
        globalCSS,
        snapPayloads: slice,
        chunk,
        staticPackage,
        assetsPackage,
        pages: pageSlice
      }));
      const payloadHash = (0, _createHash.default)(payloadString + (pageSlice ? Math.random() : ''));
      return (0, _makeRequest.default)({
        url: `${endpoint}/api/snap-requests`,
        method: 'POST',
        json: true,
        formData: {
          type: `browser-${this.browserName}`,
          targetName,
          payloadHash,
          payload: {
            options: {
              filename: 'payload.json',
              contentType: 'application/json'
            },
            value: payloadString
          }
        }
      }, {
        apiKey,
        apiSecret,
        maxTries: 5
      });
    };

    const promises = [];
    const requestIds = [];

    if (staticPackage) {
      for (let i = 0; i < this.chunks; i += 1) {
        // We allow one `await` inside the loop here to avoid POSTing all payloads
        // to the server at the same time (thus reducing load a little).
        // eslint-disable-next-line no-await-in-loop
        const {
          requestId
        } = await boundMakeRequest({
          chunk: {
            index: i,
            total: this.chunks
          }
        });

        if (asyncResults) {
          requestIds.push(requestId);
        } else {
          promises.push(waitFor({
            requestId,
            endpoint,
            apiKey,
            apiSecret
          }));
        }
      }
    } else if (pages) {
      const pagesPerChunk = Math.ceil(pages.length / this.chunks);

      for (let i = 0; i < this.chunks; i += 1) {
        const pageSlice = pages.slice(i * pagesPerChunk, i * pagesPerChunk + pagesPerChunk); // We allow one `await` inside the loop here to avoid POSTing all payloads
        // to the server at the same time (thus reducing load a little).
        // eslint-disable-next-line no-await-in-loop

        const {
          requestId
        } = await boundMakeRequest({
          pageSlice
        });

        if (asyncResults) {
          requestIds.push(requestId);
        } else {
          promises.push(waitFor({
            requestId,
            endpoint,
            apiKey,
            apiSecret
          }));
        }
      }
    } else {
      const snapsPerChunk = Math.ceil(snapPayloads.length / this.chunks);

      for (let i = 0; i < this.chunks; i += 1) {
        const slice = snapPayloads.slice(i * snapsPerChunk, i * snapsPerChunk + snapsPerChunk); // We allow one `await` inside the loop here to avoid POSTing all payloads
        // to the server at the same time (thus reducing load a little).
        // eslint-disable-next-line no-await-in-loop

        const {
          requestId
        } = await boundMakeRequest({
          slice
        });

        if (asyncResults) {
          requestIds.push(requestId);
        } else {
          promises.push(waitFor({
            requestId,
            endpoint,
            apiKey,
            apiSecret
          }));
        }
      }
    }

    if (asyncResults) {
      return requestIds;
    }

    const result = [];
    (await Promise.all(promises)).forEach(list => {
      result.push(...list);
    });
    return result;
  }

}

exports.default = RemoteBrowserTarget;